package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.RC2Parameters;

public class RC2Engine implements BlockCipher {
   private static byte[] piTable = new byte[]{-39, 120, -7, -60, 25, -35, -75, -19, 40, -23, -3, 121, 74, -96, -40, -99, -58, 126, 55, -125, 43, 118, 83, -114, 98, 76, 100, -120, 68, -117, -5, -94, 23, -102, 89, -11, -121, -77, 79, 19, 97, 69, 109, -115, 9, -127, 125, 50, -67, -113, 64, -21, -122, -73, 123, 11, -16, -107, 33, 34, 92, 107, 78, -126, 84, -42, 101, -109, -50, 96, -78, 28, 115, 86, -64, 20, -89, -116, -15, -36, 18, 117, -54, 31, 59, -66, -28, -47, 66, 61, -44, 48, -93, 60, -74, 38, 111, -65, 14, -38, 70, 105, 7, 87, 39, -14, 29, -101, -68, -108, 67, 3, -8, 17, -57, -10, -112, -17, 62, -25, 6, -61, -43, 47, -56, 102, 30, -41, 8, -24, -22, -34, -128, 82, -18, -9, -124, -86, 114, -84, 53, 77, 106, 42, -106, 26, -46, 113, 90, 21, 73, 116, 75, -97, -48, 94, 4, 24, -92, -20, -62, -32, 65, 110, 15, 81, -53, -52, 36, -111, -81, 80, -95, -12, 112, 57, -103, 124, 58, -123, 35, -72, -76, 122, -4, 2, 54, 91, 37, 85, -105, 49, 45, 93, -6, -104, -29, -118, -110, -82, 5, -33, 41, 16, 103, 108, -70, -55, -45, 0, -26, -49, -31, -98, -88, 44, 99, 22, 1, 63, 88, -30, -119, -87, 13, 56, 52, 27, -85, 51, -1, -80, -69, 72, 12, 95, -71, -79, -51, 46, -59, -13, -37, 71, -27, -91, -100, 119, 10, -90, 32, 104, -2, 127, -63, -83};
   private static final int BLOCK_SIZE = 8;
   private int[] workingKey;
   private boolean encrypting;

   private int[] generateWorkingKey(byte[] key, int bits) {
      int[] var6 = new int[128];

      int var3;
      for(var3 = 0; var3 != key.length; ++var3) {
         var6[var3] = key[var3] & 255;
      }

      int var10000;
      int var4;
      int key1;
      if ((key1 = key.length) < 128) {
         var3 = 0;
         var4 = var6[key1 - 1];

         while(true) {
            var4 = piTable[var4 + var6[var3++] & 255] & 255;
            int var5;
            var10000 = var5 = key1 + 1;
            var6[key1] = var4;
            if (var10000 >= 128) {
               break;
            }

            key1 = var5;
         }
      }

      var4 = bits + 7 >> 3;
      key = piTable;
      var10000 = var3 = 128 - var4;
      var6[var3] = key1 = key[var6[var3] & 255 >> (-bits & 7)] & 255;

      for(bits = var10000 - 1; bits >= 0; --bits) {
         var6[bits] = key1 = piTable[key1 ^ var6[bits + var4]] & 255;
      }

      byte var8 = 64;
      int[] var9 = new int[64];

      for(key1 = 0; key1 != var8; ++key1) {
         var9[key1] = var6[bits = key1 * 2] + (var6[bits + 1] << 8);
      }

      return var9;
   }

   private int rotateWordLeft(int x, int y) {
      int var3;
      return (var3 = x & '\uffff') << y | var3 >> 16 - y;
   }

   private void encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = ((in[inOff + 7] & 255) << 8) + (in[inOff + 6] & 255);
      int var6 = ((in[inOff + 5] & 255) << 8) + (in[inOff + 4] & 255);
      int var7 = ((in[inOff + 3] & 255) << 8) + (in[inOff + 2] & 255);
      int in1 = ((in[inOff + 1] & 255) << 8) + (in[inOff + 0] & 255);

      for(inOff = 0; inOff <= 16; inOff += 4) {
         var5 = this.rotateWordLeft(var5 + ((in1 = this.rotateWordLeft(in1 + (var7 & ~var5) + (var6 & var5) + this.workingKey[inOff], 1)) & ~(var6 = this.rotateWordLeft(var6 + (var5 & ~(var7 = this.rotateWordLeft(var7 + (var6 & ~in1) + (var5 & in1) + this.workingKey[inOff + 1], 2))) + (in1 & var7) + this.workingKey[inOff + 2], 3))) + (var7 & var6) + this.workingKey[inOff + 3], 5);
      }

      int[] inOff1;
      var6 = var5 + (inOff1 = this.workingKey)[(var5 = var6 + inOff1[(inOff = var7 + inOff1[(in1 += inOff1[var5 & 63]) & 63]) & 63]) & 63];

      for(var7 = 20; var7 <= 40; var7 += 4) {
         var6 = this.rotateWordLeft(var6 + ((in1 = this.rotateWordLeft(in1 + (inOff & ~var6) + (var5 & var6) + this.workingKey[var7], 1)) & ~(var5 = this.rotateWordLeft(var5 + (var6 & ~(inOff = this.rotateWordLeft(inOff + (var5 & ~in1) + (var6 & in1) + this.workingKey[var7 + 1], 2))) + (in1 & inOff) + this.workingKey[var7 + 2], 3))) + (inOff & var5) + this.workingKey[var7 + 3], 5);
      }

      int[] var11;
      var6 += (var11 = this.workingKey)[(var5 += var11[(inOff += var11[(in1 += var11[var6 & 63]) & 63]) & 63]) & 63];

      for(var7 = 44; var7 < 64; var7 += 4) {
         var6 = this.rotateWordLeft(var6 + ((in1 = this.rotateWordLeft(in1 + (inOff & ~var6) + (var5 & var6) + this.workingKey[var7], 1)) & ~(var5 = this.rotateWordLeft(var5 + (var6 & ~(inOff = this.rotateWordLeft(inOff + (var5 & ~in1) + (var6 & in1) + this.workingKey[var7 + 1], 2))) + (in1 & inOff) + this.workingKey[var7 + 2], 3))) + (inOff & var5) + this.workingKey[var7 + 3], 5);
      }

      int var8 = outOff + 0;
      out[var8] = (byte)in1;
      var8 = outOff + 1;
      out[var8] = (byte)(in1 >> 8);
      var8 = outOff + 2;
      out[var8] = (byte)inOff;
      var8 = outOff + 3;
      out[var8] = (byte)(inOff >> 8);
      var8 = outOff + 4;
      out[var8] = (byte)var5;
      var8 = outOff + 5;
      out[var8] = (byte)(var5 >> 8);
      var8 = outOff + 6;
      out[var8] = (byte)var6;
      var8 = outOff + 7;
      out[var8] = (byte)(var6 >> 8);
   }

   private void decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = ((in[inOff + 7] & 255) << 8) + (in[inOff + 6] & 255);
      int var6 = ((in[inOff + 5] & 255) << 8) + (in[inOff + 4] & 255);
      int var7 = ((in[inOff + 3] & 255) << 8) + (in[inOff + 2] & 255);
      int in1 = ((in[inOff + 1] & 255) << 8) + (in[inOff + 0] & 255);

      for(inOff = 60; inOff >= 44; inOff -= 4) {
         var5 = this.rotateWordLeft(var5, 11) - ((in1 & ~var6) + (var7 & var6) + this.workingKey[inOff + 3]);
         var6 = this.rotateWordLeft(var6, 13) - ((var5 & ~var7) + (in1 & var7) + this.workingKey[inOff + 2]);
         var7 = this.rotateWordLeft(var7, 14) - ((var6 & ~in1) + (var5 & in1) + this.workingKey[inOff + 1]);
         in1 = this.rotateWordLeft(in1, 15) - ((var7 & ~var5) + (var6 & var5) + this.workingKey[inOff]);
      }

      int var10000 = in1;
      int[] inOff1;
      int[] var10001 = inOff1 = this.workingKey;
      int var10002 = var5 -= inOff1[var6 & 63];
      int var10005 = in1;
      in1 = var6 - inOff1[var7 & 63];
      inOff = var7 - inOff1[var10005 & 63];
      var6 = var10000 - var10001[var10002 & 63];

      for(var7 = 40; var7 >= 20; var7 -= 4) {
         var5 = this.rotateWordLeft(var5, 11) - ((var6 & ~in1) + (inOff & in1) + this.workingKey[var7 + 3]);
         in1 = this.rotateWordLeft(in1, 13) - ((var5 & ~inOff) + (var6 & inOff) + this.workingKey[var7 + 2]);
         inOff = this.rotateWordLeft(inOff, 14) - ((in1 & ~var6) + (var5 & var6) + this.workingKey[var7 + 1]);
         var6 = this.rotateWordLeft(var6, 15) - ((inOff & ~var5) + (in1 & var5) + this.workingKey[var7]);
      }

      int[] var11;
      var10001 = var11 = this.workingKey;
      var10002 = var5 -= var11[in1 & 63];
      in1 -= var11[inOff & 63];
      inOff -= var11[var6 & 63];
      var6 -= var10001[var10002 & 63];

      for(var7 = 16; var7 >= 0; var7 -= 4) {
         var5 = this.rotateWordLeft(var5, 11) - ((var6 & ~in1) + (inOff & in1) + this.workingKey[var7 + 3]);
         in1 = this.rotateWordLeft(in1, 13) - ((var5 & ~inOff) + (var6 & inOff) + this.workingKey[var7 + 2]);
         inOff = this.rotateWordLeft(inOff, 14) - ((in1 & ~var6) + (var5 & var6) + this.workingKey[var7 + 1]);
         var6 = this.rotateWordLeft(var6, 15) - ((inOff & ~var5) + (in1 & var5) + this.workingKey[var7]);
      }

      int var8 = outOff + 0;
      out[var8] = (byte)var6;
      var8 = outOff + 1;
      out[var8] = (byte)(var6 >> 8);
      var8 = outOff + 2;
      out[var8] = (byte)inOff;
      var8 = outOff + 3;
      out[var8] = (byte)(inOff >> 8);
      var8 = outOff + 4;
      out[var8] = (byte)in1;
      var8 = outOff + 5;
      out[var8] = (byte)(in1 >> 8);
      var8 = outOff + 6;
      out[var8] = (byte)var5;
      var8 = outOff + 7;
      out[var8] = (byte)(var5 >> 8);
   }

   public void init(boolean encrypting, CipherParameters params) {
      this.encrypting = encrypting;
      if (params instanceof RC2Parameters) {
         RC2Engine var10000 = this;
         RC2Engine var10001 = this;
         RC2Parameters var10002 = (RC2Parameters)params;
         byte[] var3 = ((RC2Parameters)params).getKey();
         var10000.workingKey = var10001.generateWorkingKey(var3, var10002.getEffectiveKeyBits());
      } else {
         if (!(params instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to RC2 init - " + params.getClass().getName());
         }

         byte[] var4 = ((KeyParameter)params).getKey();
         this.workingKey = this.generateWorkingKey(var4, var4.length * 8);
      }

   }

   public void reset() {
   }

   public String getAlgorithmName() {
      return "RC2";
   }

   public int getBlockSize() {
      return 8;
   }

   public final int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this.workingKey != null) {
         if (inOff + 8 <= in.length) {
            if (outOff + 8 <= out.length) {
               if (this.encrypting) {
                  this.encryptBlock(in, inOff, out, outOff);
               } else {
                  this.decryptBlock(in, inOff, out, outOff);
               }

               return 8;
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("RC2 engine not initialised");
      }
   }
}
